package gs.maintainer.model

import cats.Eq
import cats.Show

/**
  * Represents the _group_ part of the artifact coordinates.
  */
opaque type ArtifactGroup = String

/**
  * Represents the _group_ part of the artifact coordinates.
  */
object ArtifactGroup:

  /**
   * Instantiate a new [[ArtifactGroup]] based on the given candidate string.
   *
   * @param candidate The string to validate.
   * @return The new [[ArtifactGroup]], or an error if the candidate was invalid.
   */
  def validate(candidate: String): Either[ModelError, ArtifactGroup] =
    if candidate.isBlank() then
      Left(ModelError.InvalidArtifactGroup(candidate))
    else
      Right(candidate)

  given CanEqual[ArtifactGroup, ArtifactGroup] = CanEqual.derived

  given Eq[ArtifactGroup] = (x, y) => x == y

  given Show[ArtifactGroup] = _.toString()

  extension (ag: ArtifactGroup)
    /**
     * @return The underlying string for this [[ArtifactGroup]].
     */
    def unwrap(): String = ag

end ArtifactGroup
