package gs.maintainer.model

import cats.Eq
import cats.Show

/**
  * Represents the _name_ part of the artifact coordinates.
  */
opaque type ArtifactName = String

/**
  * Represents the _name_ part of the artifact coordinates.
  */
object ArtifactName:

  /**
   * Instantiate a new [[ArtifactName]] based on the given candidate string.
   *
   * @param candidate The string to validate.
   * @return The new [[ArtifactName]], or an error if the candidate was invalid.
   */
  def validate(candidate: String): Either[ModelError, ArtifactName] =
    if candidate.isBlank() then
      Left(ModelError.InvalidArtifactName(candidate))
    else
      Right(candidate)

  given CanEqual[ArtifactName, ArtifactName] = CanEqual.derived

  given Eq[ArtifactName] = (x, y) => x == y

  given Show[ArtifactName] = _.toString()

  extension (an: ArtifactName)
    /**
     * @return The underlying string for this [[ArtifactName]].
     */
    def unwrap(): String = an

end ArtifactName

