package gs.maintainer.model

/**
  * Base type for all model errors.
  */
sealed trait ModelError

/**
  * Enumerates all model errors.
  */
object ModelError:

  /**
   * Produced when attempting to instantiate an [[ArtifactGroup]] and the 
   * candidate string is invalid (blank).
   *
   * @param candidate The candidate artifact group.
   */
  case class InvalidArtifactGroup(candidate: String) extends ModelError

  /**
   * Produced when attempting to instantiate an [[ArtifactName]] and the 
   * candidate string is invalid (blank).
   *
   * @param candidate The candidate artifact name.
   */
  case class InvalidArtifactName(candidate: String) extends ModelError

  /**
      * Produced when parsing a timestamp and the candidate string is not a 
      * valid timestamp emitted by `ISO_INSTANT`.
      *
      * @param candidate The candidate timestamp.
      */
  case class InvalidTimestamp(candidate: String) extends ModelError

end ModelError
